PrevRow

Enum PrevRow 

enum PrevRow {
    None,
    InPlace(usize),
    Scratch(Vec<u8>),
}
An already unfilter-ed, previous row.

The data excludes the RowFilter byte - it only includes the actual pixel data.

Variants



None

No unfiltered row.

None is the value of UnfilteringBuffer::prev_row before any row has been unfiltered (or at the start of a new interlace pass).



InPlace(usize)

Offset of UnfilteringBuffer::data_stream where the unfiltered row starts.

UnfilteringBuffer::InPlace(_) is used by unfilter_curr_row_in_place when setting UnfilteringBuffer::prev_row.



Scratch(Vec<u8>)

Separate scratch buffer containing the unfiltered row data.

UnfilteringBuffer::Scratch(_) is used by unfilter_curr_row_using_scratch_buffer when setting UnfilteringBuffer::prev_row.

Implementations

Source

impl PrevRow

fn as_slice<'a>(&'a self, buf: &'a [u8]) -> &'a [u8]

Returns the previous unfiltered row as a slice of bytes.

buf should refer to the ..current_start portion of UnfilteringBuffer::data_stream.